@@ -174,9 +174,12 @@ kernel_release (void)
 static int
 find_kernel_elf (Dwfl *dwfl, const char *release, char **fname)
 {
-  if ((release[0] == '/'
-       ? asprintf (fname, "%s/vmlinux", release)
-       : asprintf (fname, "/boot/vmlinux-%s", release)) < 0)
+  /* First try to find an uncompressed vmlinux image.  Possibly
+     including debuginfo.  */
+  if (release == NULL
+      || ((release[0] == '/'
+	   ? asprintf (fname, "%s/vmlinux", release)
+	   : asprintf (fname, "/boot/vmlinux-%s", release)) < 0))
     return -1;
 
   int fd = try_kernel_name (dwfl, fname, true);
@@ -188,6 +191,27 @@ find_kernel_elf (Dwfl *dwfl, const char *release, char **fname)
       fd = try_kernel_name (dwfl, fname, true);
     }
 
+  /* There might be a compressed vmlinuz image.  Probably without
+     debuginfo, but try to find it under the debug path also, just in
+     case.  */
+  if (fd < 0)
+    {
+      free (*fname);
+      if ((release[0] == '/'
+           ? asprintf (fname, "%s/vmlinuz", release)
+           : asprintf (fname, "/boot/vmlinuz-%s", release)) < 0)
+        return -1;
+
+      fd = try_kernel_name (dwfl, fname, true);
+      if (fd < 0 && release[0] != '/')
+	{
+	  free (*fname);
+	  if (asprintf (fname, MODULEDIRFMT "/vmlinuz", release) < 0)
+	    return -1;
+	  fd = try_kernel_name (dwfl, fname, true);
+	}
+    }
+
   return fd;
 }
 
@@ -218,6 +242,9 @@ report_kernel (Dwfl *dwfl, const char **release,
   if (unlikely (result != 0))
     return result;
 
+  if (release == NULL || *release == NULL)
+    return EINVAL;
+
   char *fname;
   int fd = find_kernel_elf (dwfl, *release, &fname);
 
@@ -273,6 +300,9 @@ report_kernel_archive (Dwfl *dwfl, const char **release,
   if (unlikely (result != 0))
     return result;
 
+  if (release == NULL || *release == NULL)
+    return EINVAL;
+
   char *archive;
   int res = (((*release)[0] == '/')
 	     ? asprintf (&archive, "%s/debug.a", *release)
@@ -493,14 +523,14 @@ intuit_kernel_bounds (Dwarf_Addr *start, Dwarf_Addr *end, Dwarf_Addr *notes)
 {
   struct read_address_state state = { NULL, NULL, 0, 0, NULL, NULL };
 
+  *notes = 0;
+
   state.f = fopen (KSYMSFILE, "r");
   if (state.f == NULL)
     return errno;
 
   (void) __fsetlocking (state.f, FSETLOCKING_BYCALLER);
 
-  *notes = 0;
-
   int result;
   do
     result = read_address (&state, start) ? 0 : -1;
@@ -695,9 +725,6 @@ dwfl_linux_kernel_report_kernel (Dwfl *dwfl)
   /* Try to figure out the bounds of the kernel image without
      looking for any vmlinux file.  */
   Dwarf_Addr notes;
-  /* The compiler cannot deduce that if intuit_kernel_bounds returns
-     zero NOTES will be initialized.  Fake the initialization.  */
-  asm ("" : "=m" (notes));
   int result = intuit_kernel_bounds (&start, &end, &notes);
   if (result == 0)
     {
